import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { Repository } from 'typeorm';
import { InjectRepository } from '@nestjs/typeorm';
import { Carrito } from './entity/carrito.entity';
import { ModuloService } from '../modulo/modulo.service';
import { InstitucionTipoCarritoService } from '../institucion-tipo-carrito/institucion-tipo-carrito.service';

@Injectable()
export class CarritoService {
  constructor(
    @InjectRepository(Carrito) private repository: Repository<Carrito>,
    private moduloService: ModuloService,
    private institucionTipoCarritoService: InstitucionTipoCarritoService,
  ) {}

  // Eliminar carrito (argumento) de este proceso. Buscar cuantos carritos tiene el modulo de ese tipo carrito y automaticamente generar carrito
  async create(id_tipo_carrito: number, id_modulo: number, carrito: string) {
    const modulo = await this.moduloService.findById(id_modulo);
    const tipoCarrito =
      await this.institucionTipoCarritoService.findByIdTipoCarito(
        id_tipo_carrito,
      );
    const nuevoCarrito = this.repository.create({
      carrito,
      modulo,
      tipoCarrito,
    });

    return this.repository
      .findOne({ carrito, modulo, tipoCarrito })
      .then((existeCarrito) => {
        if (existeCarrito)
          throw new ConflictException('Este carrito ya existe en este modulo');
        return this.repository.save(nuevoCarrito);
      })
      .then(() => ({
        message: 'Se creo correctamente el carrito en el modulo.',
      }));
  }

  findAll() {
    return this.repository.find();
  }

  findById(id_carrito: number) {
    return this.repository.findOne({ id_carrito }).then((carrito) => {
      if (!carrito) throw new NotFoundException('No existe este carrito');
      return carrito;
    });
  }

  // Toma de ejemplo la funcion que esta en /src/usuario/usuario.service.ts
  findByIdModulo(
    pagina: number,
    id_modulo: number,
    id_tipo_carrito: number,
    activo: boolean,
  ) {
    return this.repository.findOne(id_modulo).then((modulo) => {
      if (!modulo)
        throw new NotFoundException('No existen carritos es este modulo');
      return modulo;
    });
  }

  // re hacer
  async update(attrs: Partial<Carrito>) {
    const carrito = await this.findById(attrs.id_carrito);

    return this.repository
      .findOne({ carrito: attrs.carrito, modulo: carrito.modulo })
      .then((existeCarrito) => {
        if (existeCarrito)
          throw new ConflictException('Ya existe un carrito con este número.');
        Object.assign(carrito, attrs);
        return this.repository.save(carrito);
      })
      .then((_) => ({
        message: 'Se actualizo correctamente la información del carrito.',
      }));
  }
}
