import {
  Body,
  Controller,
  Get,
  Post,
  Put,
  Query,
  UseGuards,
} from '@nestjs/common';
import { UsuarioService } from './usuario.service';
import { UsuarioEscolaresDto } from './dto/usuario-escolares.dto';
import { UsuarioRegistrarDto } from './dto/usuario-registrar.dto';
import { UsuarioUpdateDto } from './dto/usuario-update.dto';
import { UsuarioUsuariosDto } from './dto/usuario-usuarios.dto';
import { UsuarioDto } from './dto/usuario.dto';
import { AuthGuard } from '@nestjs/passport';

@Controller('usuario')
export class UsuarioController {
  constructor(private usuarioService: UsuarioService) {}

  /* Pendiente por conexión */
  @Get('escolares')
  escolares(@Query() query: UsuarioEscolaresDto) {}

  @Post('registrar')
  registrar(@Body() body: UsuarioRegistrarDto) {
    return this.usuarioService.registrar(body.id_usuario, body.telefono);
  }

  @Put()
  @UseGuards(AuthGuard('jwt'))
  update(@Body() body: UsuarioUpdateDto) {
    return this.usuarioService.update(body);
  }

  @Get('usuario')
  @UseGuards(AuthGuard('jwt'))
  usuario(@Query() query: UsuarioDto) {
    return this.usuarioService.findById(Number(query.id_usuario));
  }

  @Get('usuarios')
  @UseGuards(AuthGuard('jwt'))
  usuarios(@Query() query: UsuarioUsuariosDto) {
    return this.usuarioService.findAll(query);
  }
}
