import {
  ConflictException,
  Injectable,
  NotFoundException,
} from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { Usuario } from './entity/usuario.entity';
import { Carrera } from '../carrera/entity/carrera.entity';
import { Institucion } from '../institucion/entity/institucion.entity';
import { TipoUsuario } from '../tipo-usuario/entity/tipo-usuario.entity';
import { BcryptService } from '../bcrypt/bcrypt.service';
// import { CarreraService } from '../carrera/carrera.service';
import { InstitucionService } from '../institucion/institucion.service';
import { TipoUsuarioService } from '../tipo-usuario/tipo-usuario.service';

@Injectable()
export class UsuarioService {
  constructor(
    @InjectRepository(Usuario) private repository: Repository<Usuario>,
    private bcryptService: BcryptService,
    // private carreraService: CarreraService,
    private institucionService: InstitucionService,
    private tipoUsuarioService: TipoUsuarioService,
  ) {}

  async findAll(filtros: {
    pagina: string;
    id_carrera?: string;
    id_institucion?: string;
    id_tipo_usuario?: string;
    usuario?: string;
  }) {
    const busqueda: {
      carrera?: Carrera;
      institucion?: Institucion;
      tipoUsuario?: TipoUsuario;
      usuario?: string;
    } = {};
    const institucion = filtros.id_institucion
      ? await this.institucionService.findById(Number(filtros.id_institucion))
      : null;
    const tipoUsuario = filtros.id_tipo_usuario
      ? await this.tipoUsuarioService.findById(Number(filtros.id_tipo_usuario))
      : null;
    /* Terminar cuando se tengan las fuinciones faltantes */
    // const carrera = filtros.id_carrera
    //   ? await this.carreraService.findById(Number(filtros.id_carrera))
    //   : null;

    /* buscar like */
    if (filtros.usuario) busqueda.usuario = filtros.usuario;
    if (institucion) busqueda.institucion = institucion;
    if (tipoUsuario) busqueda.tipoUsuario = tipoUsuario;
    // if (carrera) busqueda.carrera = carrera;
    /* falta página */
    console.log(busqueda);
    return this.repository.find(busqueda);
  }

  findById(id_usuario: number) {
    return this.repository.findOne({ id_usuario }).then((usuario) => {
      if (!usuario) throw new NotFoundException('No existe este usuario.');
      return usuario;
    });
  }

  findByUsuario(usuario: string) {
    return this.repository.findOne({ usuario }).then((usuario) => {
      if (!usuario) throw new NotFoundException('No existe este usuario.');
      return usuario;
    });
  }

  generarPassword = () => {
    const length = 8;
    const charset =
      'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
    const n = charset.length;
    let password = '';

    for (let i = 0; i < length; i++)
      password += charset.charAt(Math.floor(Math.random() * n));
    return password;
  };

  passwordReset(id_usuario: number) {
    const password = this.generarPassword();

    return this.findById(id_usuario)
      .then((usuario) => {
        if (!usuario.password)
          throw new ConflictException('No existe este usuario.');
        usuario.password = this.bcryptService.encriptar(password);
        /* Mandar correo con contraseña */
        return this.repository.save(usuario);
      })
      .then((_) => ({
        message: 'Se reseteo correctamente la constraseña del usuario.',
      }));
  }

  registrar(id_usuario: number, telefono: string) {
    const password = this.generarPassword();

    return this.findById(id_usuario)
      .then((usuario) => {
        if (usuario.password)
          throw new ConflictException(
            'Ya fue registrado este número de cuenta.',
          );
        usuario.telefono = telefono;
        usuario.password = this.bcryptService.encriptar(password);
        /* Mandar correo con contraseña */
        return this.repository.save(usuario);
      })
      .then((_) => ({ message: 'Se creo correctamente un usuario.' }));
  }

  update(attrs: Partial<Usuario>) {
    return this.findById(attrs.id_usuario)
      .then((usuario) => {
        if (!usuario.password)
          throw new ConflictException('No existe este usuario.');
        Object.assign(usuario, attrs);
        return this.repository.save(usuario);
      })
      .then((_) => ({
        message: 'Se actualizo correctamente la información del usuario.',
      }));
  }
}
